@@ -57,18 +57,6 @@ pub struct SortedSet <T : Ord> {
   set : SortedVec <T>
 }
 
-/// Reverse sorted vector
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[cfg_attr(all(feature = "serde", not(feature = "serde-nontransparent")),
-  serde(transparent))]
-#[derive(Clone, Debug, Eq, Ord, PartialEq, PartialOrd)]
-pub struct ReverseSortedVec <T : Ord> {
-  #[cfg_attr(feature = "serde", serde(deserialize_with = "parse_reverse_vec"))]
-  #[cfg_attr(feature = "serde",
-    serde(bound(deserialize = "T : serde::Deserialize <'de>")))]
-  vec : Vec <T>
-}
-
 #[cfg(feature = "serde")]
 fn parse_reverse_vec <'de, D, T> (deserializer : D) -> Result <Vec <T>, D::Error> where
   D : serde::Deserializer <'de>,
@@ -84,13 +72,62 @@ fn parse_reverse_vec <'de, D, T> (deserializer : D) -> Result <Vec <T>, D::Error
   }
 }
 
-/// Reverse sorted set
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[cfg_attr(all(feature = "serde", not(feature = "serde-nontransparent")),
-  serde(transparent))]
-#[derive(Clone, Debug, Eq, Ord, PartialEq, PartialOrd)]
-pub struct ReverseSortedSet <T : Ord> {
-  set : ReverseSortedVec <T>
+/// Value returned when find_or_insert is used.
+#[derive(PartialEq, PartialOrd, Eq, Ord, Debug, Hash)]
+pub enum FindOrInsert {
+  /// Contains a found index
+  Found(usize),
+
+  /// Contains an inserted index
+  Inserted(usize),
+}
+
+/// Converts from the binary_search result type into the FindOrInsert type
+impl From<Result<usize, usize>> for FindOrInsert {
+  fn from(result: Result<usize, usize>) -> Self {
+    match result {
+      Result::Ok(value) => FindOrInsert::Found(value),
+      Result::Err(value) => FindOrInsert::Inserted(value),
+    }
+  }
+}
+
+impl FindOrInsert {
+
+  /// Get the index of the element that was either found or inserted.
+  pub fn index(&self) -> usize {
+    match self {
+      FindOrInsert::Found(value) | FindOrInsert::Inserted(value) => *value
+    }
+  }
+
+  /// If an equivalent element was found in the container, get the value of
+  /// its index. Otherwise get None.
+  pub fn found(&self) -> Option<usize> {
+    match self {
+      FindOrInsert::Found(value) => Some(*value),
+      FindOrInsert::Inserted(_) => None
+    }
+  }
+
+  /// If the provided element was inserted into the container, get the value
+  /// of its index. Otherwise get None.
+  pub fn inserted(&self) -> Option<usize> {
+    match self {
+      FindOrInsert::Found(_) => None,
+      FindOrInsert::Inserted(value) => Some(*value)
+    }
+  }
+
+  /// Returns true if the element was found.
+  pub fn is_found(&self) -> bool {
+    matches!(self, FindOrInsert::Found(_))
+  }
+
+  /// Returns true if the element was inserted.
+  pub fn is_inserted(&self) -> bool {
+    matches!(self, FindOrInsert::Inserted(_))
+  }
 }
 
 //
@@ -123,11 +160,11 @@ impl <T : Ord> SortedVec <T> {
   }
   /// Find the element and return the index with `Ok`, otherwise insert the
   /// element and return the new element index with `Err`.
-  pub fn find_or_insert (&mut self, element : T) -> Result <usize, usize> {
+  pub fn find_or_insert (&mut self, element : T) -> FindOrInsert {
     self.binary_search (&element).map_err (|insert_at| {
       self.vec.insert (insert_at, element);
       insert_at
-    })
+    }).into()
   }
   #[inline]
   pub fn remove_item (&mut self, item : &T) -> Option <T> {
@@ -238,17 +275,30 @@ impl <T : Ord> SortedSet <T> {
     SortedSet { set }
   }
   /// Insert an element into sorted position, returning the order index at which
-  /// it was placed.
+  /// it was placed. If an existing item was found it will be returned.
   #[inline]
-  pub fn insert (&mut self, element : T) -> usize {
-    let _ = self.remove_item (&element);
-    self.set.insert (element)
+  pub fn replace (&mut self, mut element : T) -> (usize, Option <T>) {
+    match self.set.binary_search(&element) {
+      Ok (existing_index) => {
+        unsafe {
+          // If binary_search worked correctly, then this must be the index of a
+          // valid element to get from the vector.
+          std::mem::swap (&mut element,
+            self.set.vec.get_unchecked_mut(existing_index))
+        }
+        (existing_index, Some (element))
+      },
+      Err (insert_index) => {
+        self.set.vec.insert(insert_index, element);
+        (insert_index, None)
+      }
+    }
   }
   /// Find the element and return the index with `Ok`, otherwise insert the
   /// element and return the new element index with `Err`.
   #[inline]
-  pub fn find_or_insert (&mut self, element : T) -> Result <usize, usize> {
-    self.set.find_or_insert (element)
+  pub fn find_or_insert (&mut self, element : T) -> FindOrInsert {
+    self.set.find_or_insert (element).into()
   }
   #[inline]
   pub fn remove_item (&mut self, item : &T) -> Option <T> {
@@ -313,7 +363,7 @@ impl <T : Ord> std::ops::Deref for SortedSet <T> {
 impl <T : Ord> Extend <T> for SortedSet <T> {
   fn extend <I : IntoIterator <Item = T>> (&mut self, iter : I) {
     for t in iter {
-      let _ = self.insert (t);
+      let _ = self.find_or_insert (t);
     }
   }
 }
@@ -324,246 +374,32 @@ impl <T : Ord + Hash> Hash for SortedSet <T> {
   }
 }
 
-//
-//  impl ReverseSortedVec
-//
-
-impl <T : Ord> ReverseSortedVec <T> {
-  #[inline]
-  pub fn new() -> Self {
-    ReverseSortedVec { vec: Vec::new() }
-  }
-  #[inline]
-  pub fn with_capacity (capacity : usize) -> Self {
-    ReverseSortedVec { vec: Vec::with_capacity (capacity) }
-  }
-  /// Uses `sort_unstable_by()` to sort in place.
-  #[inline]
-  pub fn from_unsorted (mut vec : Vec <T>) -> Self {
-    vec.sort_unstable_by (|x,y| x.cmp (y).reverse());
-    ReverseSortedVec { vec }
-  }
-  /// Insert an element into (reverse) sorted position, returning the order
-  /// index at which it was placed.
-  pub fn insert (&mut self, element : T) -> usize {
-    let insert_at = match self.binary_search (&element) {
-      Ok (insert_at) | Err (insert_at) => insert_at
-    };
-    self.vec.insert (insert_at, element);
-    insert_at
-  }
-  /// Find the element and return the index with `Ok`, otherwise insert the
-  /// element and return the new element index with `Err`.
-  pub fn find_or_insert (&mut self, element : T) -> Result <usize, usize> {
-    self.binary_search (&element).map_err (|insert_at| {
-      self.vec.insert (insert_at, element);
-      insert_at
-    })
-  }
-  #[inline]
-  pub fn remove_item (&mut self, item : &T) -> Option <T> {
-    match self.vec.binary_search_by (
-      |other_item| other_item.cmp (&item).reverse()
-    ) {
-      Ok  (remove_at) => Some (self.vec.remove (remove_at)),
-      Err (_)         => None
-    }
-  }
-  /// Panics if index is out of bounds
-  #[inline]
-  pub fn remove_index (&mut self, index : usize) -> T {
-    self.vec.remove (index)
-  }
-  #[inline]
-  pub fn binary_search (&self, x : &T) -> Result <usize, usize> {
-    self.vec.binary_search_by (|y| y.cmp (&x).reverse())
-  }
-  #[inline]
-  pub fn pop (&mut self) -> Option <T> {
-    self.vec.pop()
-  }
-  #[inline]
-  pub fn clear (&mut self) {
-    self.vec.clear()
-  }
-  #[inline]
-  pub fn dedup (&mut self) {
-    self.vec.dedup();
-  }
-  #[inline]
-  pub fn dedup_by_key <F, K> (&mut self, key : F) where
-    F : FnMut (&mut T) -> K,
-    K : PartialEq <K>
-  {
-    self.vec.dedup_by_key (key);
-  }
-  #[inline]
-  pub fn drain <R> (&mut self, range : R) -> std::vec::Drain <T> where
-    R : std::ops::RangeBounds <usize>
-  {
-    self.vec.drain (range)
-  }
-  #[inline]
-  pub fn retain <F> (&mut self, f : F) where F : FnMut (&T) -> bool {
-    self.vec.retain (f)
-  }
-  /// NOTE: to_vec() is a slice method that is accessible through deref, use
-  /// this instead to avoid cloning
-  #[inline]
-  pub fn into_vec (self) -> Vec <T> {
-    self.vec
-  }
-  /// Apply a closure mutating the reverse-sorted vector and use
-  /// `sort_unstable_by()` to re-sort the mutated vector
-  pub fn mutate_vec <F, O> (&mut self, f : F) -> O where
-    F : FnOnce (&mut Vec <T>) -> O
-  {
-    let res = f (&mut self.vec);
-    self.vec.sort_unstable_by (|x,y| x.cmp (y).reverse());
-    res
-  }
-}
-impl <T : Ord> Default for ReverseSortedVec <T> {
-  fn default() -> Self {
-    Self::new()
-  }
-}
-impl <T : Ord> From <Vec <T>> for ReverseSortedVec <T> {
-  fn from (unsorted : Vec <T>) -> Self {
-    Self::from_unsorted (unsorted)
-  }
-}
-impl <T : Ord> std::ops::Deref for ReverseSortedVec <T> {
-  type Target = Vec <T>;
-  fn deref (&self) -> &Vec <T> {
-    &self.vec
-  }
-}
-impl <T : Ord> Extend <T> for ReverseSortedVec <T> {
-  fn extend <I : IntoIterator <Item = T>> (&mut self, iter : I) {
-    for t in iter {
-      let _ = self.insert (t);
-    }
-  }
-}
-impl <T : Ord + Hash> Hash for ReverseSortedVec <T> {
-  fn hash <H : Hasher> (&self, state : &mut H) {
-    let v : &Vec <T> = self.as_ref();
-    v.hash (state);
-  }
-}
-
-//
-//  impl ReverseSortedSet
-//
-
-impl <T : Ord> ReverseSortedSet <T> {
-  #[inline]
-  pub fn new() -> Self {
-    ReverseSortedSet { set: ReverseSortedVec::new() }
-  }
-  #[inline]
-  pub fn with_capacity (capacity : usize) -> Self {
-    ReverseSortedSet { set: ReverseSortedVec::with_capacity (capacity) }
-  }
-  /// Uses `sort_unstable()` to sort in place and `dedup()` to remove
-  /// duplicates.
-  #[inline]
-  pub fn from_unsorted (vec : Vec <T>) -> Self {
-    let mut set = ReverseSortedVec::from_unsorted (vec);
-    set.dedup();
-    ReverseSortedSet { set }
-  }
-  /// Insert an element into sorted position, returning the order index at which
-  /// it was placed.
-  #[inline]
-  pub fn insert (&mut self, element : T) -> usize {
-    let _ = self.remove_item (&element);
-    self.set.insert (element)
-  }
-  /// Find the element and return the index with `Ok`, otherwise insert the
-  /// element and return the new element index with `Err`.
-  #[inline]
-  pub fn find_or_insert (&mut self, element : T) -> Result <usize, usize> {
-    self.set.find_or_insert (element)
-  }
-  #[inline]
-  pub fn remove_item (&mut self, item : &T) -> Option <T> {
-    self.set.remove_item (item)
-  }
-  /// Panics if index is out of bounds
-  #[inline]
-  pub fn remove_index (&mut self, index : usize) -> T {
-    self.set.remove_index (index)
-  }
-  #[inline]
-  pub fn pop (&mut self) -> Option <T> {
-    self.set.pop()
-  }
-  #[inline]
-  pub fn clear (&mut self) {
-    self.set.clear()
-  }
-  #[inline]
-  pub fn drain <R> (&mut self, range : R) -> std::vec::Drain <T> where
-    R : std::ops::RangeBounds <usize>
-  {
-    self.set.drain (range)
-  }
-  #[inline]
-  pub fn retain <F> (&mut self, f : F) where F : FnMut (&T) -> bool {
-    self.set.retain (f)
-  }
-  /// NOTE: to_vec() is a slice method that is accessible through deref, use
-  /// this instead to avoid cloning
-  #[inline]
-  pub fn into_vec (self) -> Vec <T> {
-    self.set.into_vec()
-  }
-  /// Apply a closure mutating the sorted vector and use `sort_unstable()`
-  /// to re-sort the mutated vector and `dedup()` to remove any duplicate
-  /// values
-  pub fn mutate_vec <F, O> (&mut self, f : F) -> O where
-    F : FnOnce (&mut Vec <T>) -> O
-  {
-    let res = self.set.mutate_vec (f);
-    self.set.dedup();
-    res
-  }
-}
-impl <T : Ord> Default for ReverseSortedSet <T> {
-  fn default() -> Self {
-    Self::new()
-  }
-}
-impl <T : Ord> From <Vec <T>> for ReverseSortedSet <T> {
-  fn from (unsorted : Vec <T>) -> Self {
-    Self::from_unsorted (unsorted)
-  }
-}
-impl <T : Ord> std::ops::Deref for ReverseSortedSet <T> {
-  type Target = ReverseSortedVec <T>;
-  fn deref (&self) -> &ReverseSortedVec <T> {
-    &self.set
-  }
-}
-impl <T : Ord> Extend <T> for ReverseSortedSet <T> {
-  fn extend <I : IntoIterator <Item = T>> (&mut self, iter : I) {
-    for t in iter {
-      let _ = self.insert (t);
-    }
-  }
-}
-impl <T : Ord + Hash> Hash for ReverseSortedSet <T> {
-  fn hash <H : Hasher> (&self, state : &mut H) {
-    let v : &Vec <T> = self.as_ref();
-    v.hash (state);
-  }
-}
+/// Reverse-sorted Containers.
+///
+/// Use these containers to have the vector sorted in the reverse order of its
+/// usual comparison.
+///
+/// Note that objects going into the reverse container needs to be wrapped in
+/// std::cmp::Reverse.
+///
+/// # Examples
+///
+/// ```
+/// use std::cmp::Reverse;
+/// use sorted_vec::ReverseSortedVec;
+///
+/// let mut vec = ReverseSortedVec::<u64>::new();
+/// vec.insert(Reverse(10));
+/// vec.insert(Reverse(15));
+/// assert_eq!(vec.last().unwrap().0, 10);
+/// ```
+pub type ReverseSortedVec<T> = SortedVec<std::cmp::Reverse<T>>;
+pub type ReverseSortedSet<T> = SortedSet<std::cmp::Reverse<T>>;
 
 #[cfg(test)]
 mod tests {
   use super::*;
+  use std::cmp::Reverse;
 
   #[test]
   fn test_sorted_vec() {
@@ -572,7 +408,8 @@ mod tests {
     assert_eq!(v.insert (3), 0);
     assert_eq!(v.insert (4), 1);
     assert_eq!(v.insert (4), 1);
-    assert_eq!(v.find_or_insert (4), Ok (2));
+    assert_eq!(v.find_or_insert (4), FindOrInsert::Found (2));
+    assert_eq!(v.find_or_insert (4).index(), 2);
     assert_eq!(v.len(), 4);
     v.dedup();
     assert_eq!(v.len(), 3);
@@ -598,11 +435,12 @@ mod tests {
   #[test]
   fn test_sorted_set() {
     let mut s = SortedSet::new();
-    assert_eq!(s.insert (5), 0);
-    assert_eq!(s.insert (3), 0);
-    assert_eq!(s.insert (4), 1);
-    assert_eq!(s.insert (4), 1);
-    assert_eq!(s.find_or_insert (4), Ok (1));
+    assert_eq!(s.replace (5), (0, None));
+    assert_eq!(s.replace (3), (0, None));
+    assert_eq!(s.replace (4), (1, None));
+    assert_eq!(s.replace (4), (1, Some (4)));
+    assert_eq!(s.find_or_insert (4), FindOrInsert::Found (1));
+    assert_eq!(s.find_or_insert (4).index(), 1);
     assert_eq!(s.len(), 3);
     assert_eq!(s.binary_search (&3), Ok (0));
     assert_eq!(**SortedSet::from_unsorted (
@@ -626,61 +464,60 @@ mod tests {
   #[test]
   fn test_reverse_sorted_vec() {
     let mut v = ReverseSortedVec::new();
-    assert_eq!(v.insert (5), 0);
-    assert_eq!(v.insert (3), 1);
-    assert_eq!(v.insert (4), 1);
-    assert_eq!(v.find_or_insert (6), Err (0));
-    assert_eq!(v.insert (4), 2);
-    assert_eq!(v.find_or_insert (4), Ok (2));
+    assert_eq!(v.insert (Reverse(5)), 0);
+    assert_eq!(v.insert (Reverse(3)), 1);
+    assert_eq!(v.insert (Reverse(4)), 1);
+    assert_eq!(v.find_or_insert (Reverse(6)), FindOrInsert::Inserted (0));
+    assert_eq!(v.insert (Reverse(4)), 2);
+    assert_eq!(v.find_or_insert (Reverse(4)), FindOrInsert::Found (2));
     assert_eq!(v.len(), 5);
     v.dedup();
     assert_eq!(v.len(), 4);
-    assert_eq!(v.binary_search (&3), Ok (3));
     assert_eq!(*ReverseSortedVec::from_unsorted (
-      vec![5, -10, 99, -11, 2, 17, 10]),
-      vec![99, 17, 10, 5, 2, -10, -11]);
+      Vec::from_iter ([5, -10, 99, -11, 2, 17, 10].map (Reverse))),
+      Vec::from_iter ([99, 17, 10, 5, 2, -10, -11].map (Reverse)));
     assert_eq!(ReverseSortedVec::from_unsorted (
-      vec![5, -10, 99, -11, 2, 17, 10]),
-      vec![5, -10, 99, -11, 2, 17, 10].into());
+      Vec::from_iter ([5, -10, 99, -11, 2, 17, 10].map (Reverse))),
+      Vec::from_iter ([5, -10, 99, -11, 2, 17, 10].map (Reverse)).into());
     let mut v = ReverseSortedVec::new();
-    v.extend(vec![5, -10, 99, -11, 2, 17, 10].into_iter());
-    assert_eq!(*v, vec![99, 17, 10, 5, 2, -10, -11]);
+    v.extend([5, -10, 99, -11, 2, 17, 10].map (Reverse));
+    assert_eq!(v.as_slice(), [99, 17, 10, 5, 2, -10, -11].map (Reverse));
     let _ = v.mutate_vec (|v|{
-      v[6] = 11;
-      v[3] = 1;
+      v[6] = Reverse(11);
+      v[3] = Reverse(1);
     });
     assert_eq!(
-      v.drain(..).collect::<Vec <i32>>(),
-      vec![99, 17, 11, 10, 2, 1, -10]);
+      v.drain(..).collect::<Vec <Reverse<i32>>>(),
+      Vec::from_iter ([99, 17, 11, 10, 2, 1, -10].map (Reverse)));
   }
 
   #[test]
   fn test_reverse_sorted_set() {
     let mut s = ReverseSortedSet::new();
-    assert_eq!(s.insert (5), 0);
-    assert_eq!(s.insert (3), 1);
-    assert_eq!(s.insert (4), 1);
-    assert_eq!(s.find_or_insert (6), Err (0));
-    assert_eq!(s.insert (4), 2);
-    assert_eq!(s.find_or_insert (4), Ok (2));
+    assert_eq!(s.replace (Reverse(5)), (0, None));
+    assert_eq!(s.replace (Reverse(3)), (1, None));
+    assert_eq!(s.replace (Reverse(4)), (1, None));
+    assert_eq!(s.find_or_insert (Reverse(6)), FindOrInsert::Inserted (0));
+    assert_eq!(s.replace (Reverse(4)), (2, Some (Reverse(4))));
+    assert_eq!(s.find_or_insert (Reverse(4)), FindOrInsert::Found (2));
     assert_eq!(s.len(), 4);
-    assert_eq!(s.binary_search (&3), Ok (3));
+    assert_eq!(s.binary_search (&Reverse(3)), Ok (3));
     assert_eq!(**ReverseSortedSet::from_unsorted (
-      vec![5, -10, 99, -11, 2, 99, 17, 10, -10]),
-      vec![99, 17, 10, 5, 2, -10, -11]);
+      Vec::from_iter ([5, -10, 99, -11, 2, 99, 17, 10, -10].map (Reverse))),
+      Vec::from_iter ([99, 17, 10, 5, 2, -10, -11].map (Reverse)));
     assert_eq!(ReverseSortedSet::from_unsorted (
-      vec![5, -10, 99, -11, 2, 99, 17, 10, -10]),
-      vec![5, -10, 99, -11, 2, 99, 17, 10, -10].into());
+      Vec::from_iter ([5, -10, 99, -11, 2, 99, 17, 10, -10].map (Reverse))),
+      Vec::from_iter ([5, -10, 99, -11, 2, 99, 17, 10, -10].map (Reverse)).into());
     let mut s = ReverseSortedSet::new();
-    s.extend(vec![5, -10, 2, 99, -11, -11, 2, 17, 10].into_iter());
-    assert_eq!(**s, vec![99, 17, 10, 5, 2, -10, -11]);
+    s.extend([5, -10, 2, 99, -11, -11, 2, 17, 10].map (Reverse));
+    assert_eq!(s.as_slice(), [99, 17, 10, 5, 2, -10, -11].map (Reverse));
     let _ = s.mutate_vec (|s|{
-      s[6] = 17;
-      s[3] = 1;
+      s[6] = Reverse(17);
+      s[3] = Reverse(1);
     });
     assert_eq!(
-      s.drain(..).collect::<Vec <i32>>(),
-      vec![99, 17, 10, 2, 1, -10]);
+      s.drain(..).collect::<Vec <Reverse<i32>>>(),
+      Vec::from_iter ([99, 17, 10, 2, 1, -10].map (Reverse)));
   }
   #[cfg(feature = "serde-nontransparent")]
   #[test]
